<?php
/**
 * ldapsynch etendu de la classe bhLDAP.
 *
 * @package    abuledu
 * @subpackage connexion
 * @author     Thomas GILBERT TG (thomas.gilbert@ryxeo.com)
 * @version    1.0
 * @license GNU/GPLv2
 */
class ldapsynch extends bhLDAP {

     protected static $nDomaine = null;
     protected static $nUser = null;
     protected static $sUsername = '';

    /**
     * Synchronise les droit de l'utilisateur
     *
     * @param <integer> $nIdUser
     * @param <integer> $nIdDomaine
     * @param <string> $sUsername
     * @return <boolean>
     */
    public static function synchUser($nIdUser, $nIdDomaine, $sUsername) {
      self::$nUser = $nIdUser;
      self::$nDomaine = $nIdDomaine;
      self::$sUsername = $sUsername;
      $bResp = self::isResponsable();
      self::setPermissions(self::isAdminDomaine(), sfConfig::get('app_ldap_permission_admin_domaine'));
      self::setPermissions($bResp, sfConfig::get('app_ldap_permission_resp'));
      if ($bResp) {
          self::createGroupeResp();
      } else {
          self::createGroupe();
      }
      return true;
    }

    /**
     * Création du groupe dans la base abuledu sf s'il n'existe pas et ajoute l'utilisateur dans le groupe
     *
     */
    public static function createGroupe() {
        //groupe cn=Domain Admins
        if (self::$ldap != null) {
          $aGroupe = self::$ldap->getInfo("ou=groupes,", "(&(memberuid=".self::$sUsername."))", array('*'));
        }
        //Parcours des groupes
        for ($nCompteur=0 ; $nCompteur < $aGroupe['count']; $nCompteur++) {
            if ( $aGroupe[$nCompteur]['gidnumber'][0] > 10001) {
                $oGroupe = Doctrine::getTable('tbl_groupe')->findBySql('id_domaine=? and titre=?', array(self::$nDomaine, $aGroupe[$nCompteur]['cn'][0]));
                if ($oGroupe->count() == 0) {
                    $oGroupe = new tbl_groupe();
                    $oGroupe->setIdDomaine(self::$nDomaine);
                    $oGroupe->setTitre($aGroupe[$nCompteur]['cn'][0]);
                    $oGroupe->save();
                } else {
                  $oGroupe = $oGroupe[0];
                }
                //Vérifie que l'utilisateur fait partie du groupe
                $oGroupeUser = Doctrine::getTable('tbl_groupeuser')->findBySql('id_sfguarduser=? and id_groupe=?', array(self::$nUser, $oGroupe->getId()));
                if ($oGroupeUser->count() == 0) {
                    $oGroupeUser = new tbl_groupeuser();
                    $oGroupeUser->setIdSfguarduser(self::$nUser);
                    $oGroupeUser->setIdGroupe($oGroupe->getId());
                    $oGroupeUser->save();
                }
            }
        }
    }

    /**
     *  Création du groupe (en fonction du groupe responsable) dans la base abuledu sf s'il n'existe pas et ajoute l'utilisateur dans le groupe
     */
    public static function createGroupeResp() {
        if (self::$ldap != null) {
          $aGroupe = self::$ldap->getInfo("ou=groupes,", "(&(memberuid=".self::$sUsername."))", array('*'));
        }
        //Parcours des groupes
        for ($nCompteur=0 ; $nCompteur < $aGroupe['count']; $nCompteur++) {
            $aGroupeNom = explode('-', $aGroupe[$nCompteur]['cn'][0]);
            if ($aGroupeNom[0] == 'resp') {
                $oGroupe = Doctrine::getTable('tbl_groupe')->findBySql('id_domaine=? and titre=?', array(self::$nDomaine, $aGroupeNom[1]));
                if ($oGroupe->count() == 0) {
                    $oGroupe = new tbl_groupe();
                    $oGroupe->setIdDomaine(self::$nDomaine);
                    $oGroupe->setTitre($aGroupeNom[1]);
                    $oGroupe->save();
                } else {
                  $oGroupe = $oGroupe[0];
                }
                //Vérifie que l'utilisateur fait partie du groupe
                $oGroupeUser = Doctrine::getTable('tbl_groupeuser')->findBySql('id_sfguarduser=? and id_groupe=?', array(self::$nUser, $oGroupe->getId()));
                if ($oGroupeUser->count() == 0) {
                    $oGroupeUser = new tbl_groupeuser();
                    $oGroupeUser->setIdSfguarduser(self::$nUser);
                    $oGroupeUser->setIdGroupe($oGroupe->getId());
                    $oGroupeUser->save();
                }

            }
        }
    }

    /**
     * Vérifie si l'utilisateur est responsable
     *
     * @return <boolean> vrai si responsable
     */
    public static function isResponsable() {
       //groupe resp-*
        if (self::$ldap != null) {
          $aResp = self::$ldap->getInfo("ou=groupes,", "(&(cn=".sfConfig::get('app_ldap_resp_group').")(memberuid=tgilbert))", array('*'));

          if ($aResp['count'] > 0) {
              return true;
          }
        }
        return false;

    }

    /**
     * Ajoute ou supprime les droits de l'utilisateur
     * @param <boolean> $bActiver vrai si ajouter faux si supprime
     * @param <string> $sType type de droit admin ou responsable
     */
    public static function setPermissions($bActiver, $sType) {
      //Récupère le groupe admin domaine
      $oAdminDomaine = Doctrine::getTable('sfGuardGroup')->findOneBy('name', $sType);
      $oSfGroupUser = Doctrine::getTable('sfGuardUserGroup')->findBySql('user_id = ? and group_id =?', array(self::$nUser, $oAdminDomaine->getId()));
      if ($oSfGroupUser->count() == 0 && $bActiver == true) {
         $oSfGroupUser = new sfGuardUserGroup();
         $oSfGroupUser->setUserId(self::$nUser);
         $oSfGroupUser->setGroupId($oAdminDomaine->getId());
         $oSfGroupUser->save();
         //Récupère la liste des permissions add
      } elseif ($oSfGroupUser->count() > 0 && $bActiver == false) {
        $oSfGroupUser[0]->delete();
      }
    }

    /**
     * Vérifie si l'utilisateur est administrateur du domaine
     * @return <boolean> vrai si admin
     */
    public static function isAdminDomaine() {
        //groupe cn=Domain Admins
        if (self::$ldap != null) {
          $aAdmin = self::$ldap->getInfo("ou=groupes,", "(&(cn=".sfConfig::get('app_ldap_admin_group').")(memberuid=".self::$sUsername."))", array('*'));
          if ($aAdmin['count'] > 0) {
              return true;
          }
        }
        return false;
    }

    /**
     * Récupère l'information d'un champ de l'utilidateur dans la base LDAP
     *
     * @param <string> $psField Nom du champ
     * @param <string> $psUsername Nom de l'utilisateur
     * @return <string> Valeur dans du champ dans la base LDAP
     */
    public static function getFieldUsername($psField, $psUsername) {
        if (self::$ldap != null) {
          $aInfo = self::$ldap->getInfo("ou=utilisateurs,", "(&(uid=".$psUsername."))", array($psField));
          if ($aInfo['count'] > 0) {
            if ($aInfo[0][$psField]['count'] > 0) {
              return $aInfo[0][$psField][0];
            }
          }
        }
        return '';

    }
}